/* FILE: basedriver.h            -*-Mode: c++-*-
 *
 * Example concrete Oxs_Driver class.
 *
 */

#ifndef _OXS_BASEDRIVER
#define _OXS_BASEDRIVER

#include "simstate.h"
#include "key.h"
#include "driver.h"

/* End includes */

class Oxs_BaseDriver:public Oxs_Driver {
private:
  Oxs_Key<Oxs_SimState> initial_state;
  REAL8m min_timestep;          // Seconds
  REAL8m max_timestep;          // Seconds
  REAL8m stopping_dm_dt;        // Radians/sec
  UINT4m number_of_stages;      // 0 => unlimited
  UINT4m stage_iteration_limit; // 0 => no limit
  UINT4m total_iteration_limit; // 0 => no limit
public:
  Oxs_BaseDriver(const char* name,    // Child instance id
		Oxs_Director* newdtr, // App director
                 Tcl_Interp* safe_interp, // Safe interpreter
		const char* argstr);  // MIF input block parameters
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  virtual ~Oxs_BaseDriver();

  // Generic interface
  virtual Oxs_ConstKey<Oxs_SimState> GetInitialState() const
  { return initial_state; }
  virtual BOOL Done(const Oxs_SimState& state,
    Tcl_Interp*, int, char**) const;
  virtual BOOL StageDone(const Oxs_SimState& state,
    Tcl_Interp*, int, char**) const;
  virtual BOOL Return(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const;
  virtual void NewState(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const;

  virtual void FillStatePartA(const Oxs_SimState& old_state,
			      Oxs_SimState& new_state) const;
  virtual void FillStatePartB(Oxs_SimState& work_state) const;
};

#endif // _OXS_BASEDRIVER
