/* FILE: cubicdemag.h            -*-Mode: c++-*-
 *
 * Average H demag field across cubic cells.
 *
 */

#ifndef _OXS_CUBICDEMAG
#define _OXS_CUBICDEMAG

#include "depkey.h"
#include "energy.h"
#include "fft.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "cubicmesh.h"

/* End includes */

class Oxs_CubicDemag:public Oxs_Energy {
private:
  UINT4m xdim,ydim,zdim,totalsize;  // Natural size
  UINT4m pxdim,pydim,pzdim,ptotalsize;  // Padded size
  Oxs_DepKey<Oxs_CubicMesh> meshkey;
  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  Mx, My, Mz
  // and Hcomp are used as temporary space to convert M into
  // H.
  MyComplex *A00,*A01,*A02,*A11,*A12,*A22;
  MyComplex *Mx,*My,*Mz,*Hcomp;
  FFT3D fft; // All transforms are same size, so we need
  /// only one FFT3D object.

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory();
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  Oxs_CubicDemag(const char* name,     // Child instance id
		 Oxs_Director* newdtr, // App director
                 Tcl_Interp* safe_interp, // Safe interpreter
		 const char* argstr);  // MIF input block parameters
  virtual ~Oxs_CubicDemag() { ReleaseMemory(); }

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_CUBICDEMAG
