/* FILE: cubicmesh.h            -*-Mode: c++-*-
 *
 * Cell centered cubic mesh, derived from Oxs_Mesh class.
 *
 */

#ifndef _OXS_CUBICMESH
#define _OXS_CUBICMESH

#include "mesh.h"

/* End includes */


class Oxs_CubicMesh : public Oxs_Mesh {
private:
  ThreeVector base;
  REAL8m cellsize;
  UINT4m xdim,ydim,zdim;
  UINT4m xydim; // Convenience value: xdim*ydim
  UINT4m elementcount;  // Convenience value: xdim*ydim*zdim
public:
  // Oxs_Ext interface
  Oxs_CubicMesh
  (const char* name,     // Child instance id
   Oxs_Director* newdtr, // App director
   Tcl_Interp* safe_interp, // Safe interpreter
   const char* argstr);  // MIF input block parameters
  virtual ~Oxs_CubicMesh() {}
  virtual BOOL Init() { return 1; }
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.

  // Mesh interface
  virtual void GetBoundingBox(Oxs_Box& bbox) const;
  virtual UINT4m Size() const { return elementcount; }
  virtual REAL8m TotalVolume() const
    { return cellsize*cellsize*cellsize*elementcount; }
  virtual REAL8m Volume(UINT4m) const
    { return cellsize*cellsize*cellsize; }
  virtual void Center(UINT4m index,ThreeVector& location) const;


  // File output.  Returns 0 on success.
  virtual int WriteOvf
  (const char* filename,  // Name of output file
   const char* title,     // Long filename or title
   const char* desc,      // Description to embed in output file
   const char* meshtype,  // Either "rectangular" or "irregular"
   const char* datatype,  // Either "binary" or "text"
   const char* precision, // For binary, "4" or "8";
                          //  for text, a printf-style format
   const Oxs_MeshValue<ThreeVector>* vec,  // Vector array
   const Oxs_MeshValue<REAL8m>* scale=NULL // Optional scaling for vec
   /// Set scale to NULL to use vec values directly.
   ) const;

  // Cubic mesh specific functions.
  // IMPORTANT NOTE: Client routines can assume that the storage
  // order is first on x, second on y, third on z (Fortran order),
  // i.e., (0,0,0) (1,0,0) ... (xdim-1,0,0) (0,1,0) (1,1,0) ...
  // (xdim-1,ydim-1,0) (0,0,1) (1,0,1) ... (xdim-1,ydim-1,zdim-1)
  // This locks down the representation in this function, but
  // allows improved access in the clients.  If we need to change
  // representations, we can always introduce a new cubic mesh
  // class. ;^)
  REAL8m EdgeLength() const { return cellsize; }
  UINT4m XDim() const { return xdim; }
  UINT4m YDim() const { return ydim; }
  UINT4m ZDim() const { return zdim; }

  // Use the following function to convert a triplet index (x,y,z)
  // to the corresponding address used to access linear MeshValue
  // arrays.
  inline UINT4m Index(UINT4m x,UINT4m y,UINT4m z) const {
    return x+y*xdim+z*xydim;
  }
};

#endif // _OXS_CUBICMESH

