/* FILE: eulerevolve.h                 -*-Mode: c++-*-
 *
 * Concrete evolver class, using simple forward Euler steps
 *
 */

#ifndef _OXS_EULEREVOLVE
#define _OXS_EULEREVOLVE

#include "driver.h"
#include "evolver.h"
#include "key.h"

/* End includes */

class Oxs_EulerEvolve:public Oxs_Evolver {
private:
  // The following evolution constants are uniform for now.  These
  // should be changed to arrays in the future.
  REAL8m gamma;  // Landau-Lifschitz gyromagnetic ratio
  REAL8m alpha;  // Landau-Lifschitz damping coef

  // next_timestep is based on the error from the last step.  If
  // there is no last step (either because this is the first step,
  // or because the last state handled by this routine is different
  // from the incoming current_state), then timestep is calculated
  // so that max_dm_dt * timestep = start_dm.
  REAL8m start_dm;
  REAL8m next_timestep;

  // Data cached from last state
  UINT4m state_id;
  REAL8m max_dm_dt;
  REAL8m dE_dt;
  Oxs_MeshValue<ThreeVector> dm_dt;
  Oxs_MeshValue<REAL8m> energy;

  // Scratch space
  Oxs_MeshValue<REAL8m> new_energy;
  Oxs_MeshValue<ThreeVector> mxH;

  void Calculate_dm_dt
  (const Oxs_Mesh& mesh_,
   const Oxs_FixedScalarField& Ms_,
   const Oxs_MeshValue<ThreeVector>& mxH_,
   const Oxs_MeshValue<ThreeVector>& spin_,
   Oxs_MeshValue<ThreeVector>& dm_dt_,
   REAL8m& max_dm_dt_,REAL8m& dE_dt_) const;
  /// Imports: mesh_, Ms_, mxH_, spin_
  /// Exports: dm_dt_, max_dm_dt_, dE_dt_

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  Oxs_EulerEvolve(const char* name,     // Child instance id
		  Oxs_Director* newdtr, // App director
                  Tcl_Interp* safe_interp, // Safe interpreter
		  const char* argstr);  // MIF input block parameters
  virtual ~Oxs_EulerEvolve();

  virtual  BOOL
  Step(const Oxs_Driver& driver,
       Oxs_ConstKey<Oxs_SimState> current_state,
       Oxs_Key<Oxs_SimState>& next_state);
  // Returns true if step was successful, false if
  // unable to step as requested.
};

#endif // _OXS_EULEREVOLVE
