/* FILE: randomfixedvectorfield.cc     -*-Mode: c++-*-
 *
 * Random Fixed Vector Field, derived from Oxs_FixedVectorField class.
 *
 */

#include "oc.h"
#include "nb.h"

#include "threevector.h"
#include "meshvalue.h"
#include "director.h"
#include "randomfixedvectorfield.h"
#include "energy.h"		// Needed to make MSVC++ 5 happy

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_RandomFixedVectorField);

/* End includes */


// Constructor
Oxs_RandomFixedVectorField::Oxs_RandomFixedVectorField(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  Tcl_Interp* safe_interp, // Safe interpreter
  const char* argstr)   // MIF input block parameters
  : Oxs_FixedVectorField(name,newdtr,safe_interp,argstr)
{
  // Process arguments
  CheckInitValueParamCount("minnorm",1);
  REAL8m minnorm=Nb_Atof((*FindInitValue("minnorm"))[0].c_str());
  DeleteInitValue("minnorm");

  CheckInitValueParamCount("maxnorm",1);
  REAL8m maxnorm=Nb_Atof((*FindInitValue("maxnorm"))[0].c_str());
  DeleteInitValue("maxnorm");


  CheckInitValueParamCount("mesh",1);
  Oxs_DepKey<Oxs_Ext> key
    = director->FindExtObject((*FindInitValue("mesh"))[0]);
  Oxs_Mesh* mesh = dynamic_cast<Oxs_Mesh*>(key.GetPtr());
  if(mesh==NULL) {
    string msg="No Oxs_Mesh object found with name "
      + (*FindInitValue("mesh"))[0];
    throw Oxs_Ext::Error(msg.c_str());
  }
  meshkey.Set(mesh);
  DeleteInitValue("mesh");

  VerifyAllInitArgsUsed();

  size=mesh->Size();
  vec.AdjustSize(mesh);
  if(maxnorm==minnorm) {
    // No spread in magnitudes
    for(UINT4m i=0;i<size;i++) {
      vec[i].Random(maxnorm);
    }
  } else {
    // Vary both direction and magnitude
    REAL8m normrange=maxnorm-minnorm;
    for(UINT4m i=0;i<size;i++) {
      REAL8m mag = normrange*Oc_UnifRand()+minnorm;
      vec[i].Random(mag);
    }
  }
}

BOOL Oxs_RandomFixedVectorField::Init()
{ return 1; }

Oxs_RandomFixedVectorField::~Oxs_RandomFixedVectorField()
{}

ThreeVector
Oxs_RandomFixedVectorField::operator[](UINT4m index) const
{
  if(index>=size) {
    char buf[512];
    Oc_Snprintf(buf,sizeof(buf),
		"In object %s, "
		"Oxs_RandomFixedVectorField::operator[] const: "
		"Array out-of-bounds; index=%u, size=%u",
		InstanceName(),index,size);
    throw Oxs_Ext::Error(buf);
  }
  return vec[index];
}


