/* FILE: uniformfixedvectorfield.cc     -*-Mode: c++-*-
 *
 * Uniform Fixed Vector Field, derived from Oxs_FixedVectorField class.
 *
 */

#include "oc.h"
#include "nb.h"

#include "uniformfixedvectorfield.h"

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_UniformFixedVectorField);

/* End includes */


// Constructor
Oxs_UniformFixedVectorField::Oxs_UniformFixedVectorField(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  Tcl_Interp* safe_interp, // Safe interpreter
  const char* argstr)   // MIF input block parameters
  : Oxs_FixedVectorField(name,newdtr,safe_interp,argstr)
{
  // Process arguments
  CheckInitValueParamCount("vector",3);
  const vector<string>& vstr = *FindInitValue("vector");
  vec.x=Nb_Atof(vstr[0].c_str());
  vec.y=Nb_Atof(vstr[1].c_str());
  vec.z=Nb_Atof(vstr[2].c_str());
  DeleteInitValue("vector");

  const vector<string>* nstr = FindInitValue("norm");
  if(nstr!=NULL) {
    CheckInitValueParamCount("norm",1);
    vec.SetMag(Nb_Atof((*nstr)[0].c_str()));
    DeleteInitValue("norm");
  }
  VerifyAllInitArgsUsed();
}

BOOL Oxs_UniformFixedVectorField::Init()
{ return 1; }

Oxs_UniformFixedVectorField::~Oxs_UniformFixedVectorField()
{}
