# FILE: oxsolve.tcl
#
# An OOMMF solver interface for the oxs shell.

# Support libraries
package require Oc 1.1
package require Oxs 1.2
package require Net

Oc_IgnoreTermLoss  ;# Try to keep going, even if controlling terminal
## goes down.

Oc_Main SetAppName OXSolve
Oc_Main SetVersion 1.2.0.0
regexp \\\044Date:(.*)\\\044 {$Date: 2000/08/24 02:20:16 $} _ date
Oc_Main SetDate [string trim $date]
Oc_Main SetAuthor [Oc_Person Lookup dgp]
Oc_Main SetHelpURL [Oc_Url FromFilename [file join [file dirname \
        [file dirname [file dirname [Oc_DirectPathname [info \
        script]]]]] doc userguide Extensible_Micromagnetic_Solver.html]]

Oc_CommandLine Option restart {
	{flag {expr {![catch {expr {$flag && $flag}}]}} {= 0 or 1}}
    } {
	global restart_option; set restart_option $flag
} {1 => use <basename>.log file to restart simulation}
set restart_option 0

Oc_CommandLine Parse $argv

set gui {
    package require Oc 1.1
    package require Tk
    package require Ow
    wm withdraw .
}
append gui "[list Oc_Main SetAppName [Oc_Main GetAppName]]\n"
append gui "[list Oc_Main SetVersion [Oc_Main GetVersion]]\n"
append gui "[list Oc_Main SetPid [pid]]\n"
append gui {

regexp \\\044Date:(.*)\\\044 {$Date: 2000/08/24 02:20:16 $} _ date
Oc_Main SetDate [string trim $date]

# This won't cross different OOMMF installations nicely
Oc_Main SetAuthor [Oc_Person Lookup dgp]

}
# This might not cross nicely either
append gui {Oc_Main SetHelpURL [Oc_Url FromFilename }
append gui [list [file join [file dirname [file dirname [file dirname \
	[Oc_DirectPathname [info script]]]]] doc userguide \
	2D_Micromagnetic_Solver_mmS.html]]]\n
append gui {
set menubar .mb
foreach {fmenu hmenu} [Ow_MakeMenubar . $menubar File Help] {}
$fmenu add command -label "Load..." \
	-command [list LoadButton $fmenu "Load..."] -underline 0
$fmenu add command -label "Show Console" -command { console show } -underline 0
$fmenu add command -label "Close Interface" -command { exit } -underline 0
$fmenu add command -label "Exit OXSolve" -command { appexit; exit } -underline 1
Ow_StdHelpMenu $hmenu
proc LoadButton { btn item } {
    if { [string match disabled [$btn entrycget $item -state]] } {
        return
    }
    Ow_FileDlg New dialog -callback LoadCallback \
	    -dialog_title "[Oc_Main GetInstanceName]: Load Problem" \
            -selection_title "Load MIF File..." \
            -select_action_id LOAD \
	    -filter *.mif \
	    -file_must_exist 1 \
	    -delete_callback_arg [list $btn entryconfigure $item -state normal]
    $btn entryconfigure $item -state disabled
}
proc LoadCallback { widget actionid args } {
    global problem
    if {[string match DELETE $actionid]} {
	# This really is the way it has to be!  The quoting on $args
	# is just plain weird.
        eval [join $args]
        return
    }
    if {[string match LOAD $actionid]} {
	set problem [$widget GetFilename]
    } else {
        return "ERROR (proc LoadCallBack): Invalid actionid: $actionid"
    }
    # The following [return] must be here so the dialog will close 
    return
}

set menuwidth [Ow_GuessMenuWidth $menubar]
set brace [canvas .brace -width $menuwidth -height 0 -borderwidth 0 \
        -highlightthickness 0]
pack $brace -side top

wm title . [Oc_Main GetInstanceName]
wm iconname . [wm title .]
Ow_SetIcon .
wm deiconify .

if {[package vcompare [package provide Tk] 8] >= 0 \
        && [string match windows $tcl_platform(platform)]} {
    # Windows doesn't size Tcl 8.0 menubar cleanly
    Oc_DisableAutoSize .
    wm geometry . "${menuwidth}x0"
    update
    wm geometry . {}
    Oc_EnableAutoSize .
    update
}

share problem
set probframe [frame .problem ]
pack [label $probframe.l -text Problem: -anchor w] -side left -fill x
pack [label $probframe.val -textvariable problem -anchor e] -side right -fill x
pack $probframe -side top -fill x 

set mf [frame .monitor]

share status
set statframe [frame $mf.status]
pack [label $statframe.l -text Status: -anchor w] -side left -fill x
pack [label $statframe.val -textvariable status -anchor e] -side right -fill x
pack $statframe -side top -fill x

share stage
set stageframe [frame $mf.stage]
pack [label $stageframe.l -text Stage: -anchor w -width 18] -side left -fill x
pack [label $stageframe.val -textvariable stage -anchor e] -side right -fill x
pack $stageframe -side top -fill x

share step
set stepframe [frame $mf.step]
pack [label $stepframe.l -text Step: -anchor w] -side left -fill x
pack [label $stepframe.val -textvariable step -anchor e] -side right -fill x
pack $stepframe -side top -fill x

pack $mf -side left -fill both -expand 1

set interface_state disabled
share interface_state

set oframe [frame .outputs]
pack [label $oframe.iolabel -text "Interactive Output"] -side top -fill x
set vthreadframe [frame $oframe.vthreads]
pack [button $vthreadframe.mag -text Magnetization -state $interface_state \
	-command {remote OutputMagnetization}] -side top -fill x
trace variable interface_state w "[list $vthreadframe.mag configure \
	-state] \[uplevel #0 set interface_state] ;#"

share vectorFieldList
share vflbsel
Ow_ListBox New vflb $vthreadframe.l \
	-height 3 \
	-variable vflbsel \
	-selectmode multiple
pack [$vflb Cget -winpath] -side top -fill x
trace variable vectorFieldList w \
	{$vflb Remove 0 end; eval [list $vflb Insert 0] $vectorFieldList ;# }
pack $vthreadframe -side right -fill x -expand 1

set dthreadframe [frame $oframe.threads]
pack [button $dthreadframe.dt -text "Data Table" -state $interface_state \
	-command {remote OutputDataTable}] -side top -fill x
trace variable interface_state w "[list $dthreadframe.dt configure \
	-state] \[uplevel #0 set interface_state] ;#"

share dataTableList
share dtlbsel
Ow_ListBox New dtlb $dthreadframe.l \
	-height 3 \
	-variable dtlbsel \
	-selectmode multiple
pack [$dtlb Cget -winpath] -side top -fill x
trace variable dataTableList w \
        {$dtlb Remove 0 end; eval [list $dtlb Insert 0] $dataTableList ;# }
pack $dthreadframe -side right -fill x -expand 1
pack $oframe -side right -fill x -expand 1

set bf [frame .buttons]
pack [button $bf.reload -text Reload -command {set problem $problem} \
	-state $interface_state] -fill x
pack [button $bf.reset -text Reset -command {remote Reset} \
	-state $interface_state] -fill x
pack [button $bf.run -text Run -command {set status Run} \
	-state $interface_state] -fill x
pack [button $bf.pause -text Pause -command {set status Pause} \
	-state $interface_state] -fill x

pack $bf -side left -fill both -expand 1

trace variable interface_state w {
    foreach _ [winfo children .buttons] {
	$_ configure -state [uplevel #0 set interface_state];
    } ;# }

share errorstate
trace variable errorstate w {DisplayError ;# }
proc DisplayError {} {
    global errorstate errorInfo errorCode
    if {[string match {} $errorstate]} {return}
    foreach {s t m errorInfo errorCode} $errorstate {break}
    Oc_Log Log "$m\n($s $t)" error
}

}

if {[Oc_Main HasTk]} {
    wm withdraw .        ;# "." is just an empty window.
    package require Ow
    Ow_SetIcon .
} else {
    # Set up to write log messages to oommf/mmsolve2d.errors.
    # Note that the solver class will later re-direct this to its own log
    # file.
    FileLogger SetFile \
	    [file join [Oc_Main GetOOMMFRootDir] oxsolve.errors]
    Oc_Log SetLogHandler [list FileLogger Log] panic Oc_Log
    Oc_Log SetLogHandler [list FileLogger Log] error Oc_Log
    Oc_Log SetLogHandler [list FileLogger Log] warning Oc_Log
    Oc_Log SetLogHandler [list FileLogger Log] info Oc_Log
    Oc_Log SetLogHandler [list FileLogger Log] panic
    Oc_Log SetLogHandler [list FileLogger Log] error
    Oc_Log SetLogHandler [list FileLogger Log] warning
    Oc_Log SetLogHandler [list FileLogger Log] info

    Oc_EventHandler New _ FileLogger Log {
	set errorstate [list %src %type %msg %errorinfo %errorcode]
	set errorstate ""
    }
}

proc every {ms script} {
    uplevel #0 $script
    after $ms [info level 0]
}

trace variable problem w LoadProblem
Oxs_Mif New mif
proc LoadProblem {args} {
   set status Loading...
   after idle {$mif Read $problem; Reset}
}
Oc_EventHandler New _ Oc_Main Exit [list $mif Delete]
Oc_EventHandler New _ Oc_Main Exit {Oc_EventHandler Generate Oxs_Run Cleanup}

proc Reset {} {
    global status stage step
    Oc_EventHandler Generate Oxs_Run Cleanup
    set stage ""
    set step ""
    set status Initializing...
    after idle {
	Oxs_ProbInit $mif
	set status Pause
	Oc_EventHandler Generate Oxs_Run Initialized
    }
}

set status Pause
trace variable status w [list ChangeStatus $status]
proc ChangeStatus {old args} {
    global status interface_state
    if {[string match $old $status]} {return}
    trace vdelete status w [list ChangeStatus $old]
    trace variable status w [list ChangeStatus $status]
    switch -exact -- $status {
	Run {after idle every idle Oxs_Run; set interface_state normal}
	Pause {after cancel every idle Oxs_Run; set interface_state normal}
	Done {after cancel every idle Oxs_Run; set interface_state normal}
	default {set interface_state disabled}
    }
}

Oc_EventHandler Bindtags Oxs_Run [list Oxs_Run]
Oc_EventHandler New _ Oxs_Run NewState [list set step %step]
Oc_EventHandler New _ Oxs_Run StageDone [list set stage %stage]
Oc_EventHandler New _ Oxs_Run Done [list set status Done]
Oc_EventHandler New _ Oxs_Run Done \
	[list Oc_EventHandler Generate Oxs_Run Cleanup]
Oc_EventHandler New _ Oxs_Run Initialized \
	[list Oc_EventHandler New _ Oxs_Run Cleanup \
		[list RunDoneNotify] -oneshot 1]

proc RunDoneNotify {} {
    # Send end-of-run message to each thread supporting
    # the DataTable protocol
    set eormsg [list [list "@Filename:" {} 0]]
    foreach thread [Net_Thread Instances] {
        if {[string match DataTable [$thread Protocol]]} {
            $thread Send DataTable $eormsg
        }
    }
}

proc OutputMagnetization {} {
    global vflbsel threadMap
    foreach sel $vflbsel {
	Oc_TempFile New t -stem oxs[pid]-mag -extension .omf
	set tempname [$t AbsoluteName]
	$t Delete
	set realname [Oxs_Run DumpMagnetization $tempname oxs-demo]
	$threadMap($sel) Send datafile [list $tempname $realname]
    }
}

proc OutputDataTable {} {
    global dtlbsel threadMap
    foreach sel $dtlbsel {
	set triples [Oxs_Run MakeDataTable]
	$threadMap($sel) Send DataTable $triples
    }
}

set vectorFieldList {}
set dataTableList {}
Oc_EventHandler New _ Net_Thread Ready [list AddThreadToList %object]
Oc_EventHandler New _ Net_Thread Delete [list DeleteThreadFromList %object]
proc AddThreadToList {obj} {
    global threadMap
    switch -exact -- [$obj Protocol] {
	DataTable {upvar #0 dataTableList list}
	vectorField {upvar #0 vectorFieldList list}
	default {$obj Delete; return}
    }
    set pid [$obj Cget -pid]
#    regexp {[0-9]+} $pid pid
    set name [$obj Cget -alias]<$pid>
    set threadMap($name) $obj
    lappend list $name
}
proc DeleteThreadFromList {obj} {
    global threadMap
    switch -exact -- [$obj Protocol] {
	DataTable {upvar #0 dataTableList list}
	vectorField {upvar #0 vectorFieldList list}
	default {return}
    }
    set pid [$obj Cget -pid]
#    regexp {[0-9]+} $pid pid
    set name [$obj Cget -alias]<$pid>
    catch {unset threadMap($name)}
    set idx [lsearch -exact $list $name]
    set list [lreplace $list $idx $idx]
}

##########################################################################
# Our server offers GeneralInterface services -- start it
##########################################################################
Net_Protocol New protocol -name "OOMMF GeneralInterface protocol 0.0"
$protocol AddMessage start HasGui {} {
    return [list start [list 0 2]]
}
$protocol AddMessage start GetGui {} \
	[list return [list start [list 0 $gui]]]
$protocol AddMessage start Eval {cmd args} {
    set code [catch {uplevel #0 $args} result]
    if {[llength $cmd]} {
	lappend cmd $code $result
	if {$code == 1} {
	    global errorInfo errorCode
	    lappend cmd $errorInfo $errorCode 
	}
	$connection Tell Result $cmd
    }
    return [list start [list 0 ""]]
}
$protocol AddMessage start Share {local remote trace} {
    global $local

    # The script to be run in the client, when $local is written here
    # %s holds the place for the [set] command to go along with the
    # new value.
    set ctraceFmt "[list trace vdelete $remote w $trace]; %s;\
	    [list trace variable $remote w $trace] ;#"

    # A common prefix for the local write trace, and the script which
    # only starts write traces in the client.  The latter is used
    # when the variable $local does not exist.
    set pfx "[list $connection Tell Result] \[[list format $ctraceFmt]"

    # The script to be run here, when $local is written.  It substitutes
    # the [set] command into $ctraceFmt, then sends the script off to be
    # evaluated in the client.
    set strace "$pfx \[[list list set $remote] \[[list uplevel #0\
	    set $local]]]] ;#"

    # If $local already has a value; initialize the client's copy to
    # also have that value.  If not, just set write traces in the client.
    if {[info exists $local]} {eval $strace} else {eval "$pfx {}]"}

    # Set the trace, so changes here get reflected in the client.
    trace variable $local w $strace

    # Make sure this write trace is destroyed if we ever lose contact
    # with the client
    regsub -all -- % $strace %% strace
    Oc_EventHandler New _ $connection Delete \
	    [list trace vdelete $local w $strace] 
    return [list start [list 0 ""]]
}
$protocol AddMessage start delete {} {
    global protocol
    return [list start [list 0 [after 100 $protocol Delete]]]
}
Oc_EventHandler New _ $protocol Delete [list after 2000 exit] -oneshot 1
Net_Server New server -protocol $protocol -alias OXSolve
$server Start 0

##########################################################################
# Track the threads known to the account server
#	code mostly cribbed from mmLaunch 
#	good candidate for library code?
##########################################################################
# Get Thread info from account server:
proc AccountReady {acct} {
    Oc_MakeNice
    set qid [$acct Send threads]
    Oc_EventHandler New _ $acct Reply$qid [list GetThreadsReply $acct] \
        -groups [list $acct]
}
proc GetThreadsReply { acct } {
    # Set up to receive NewThread messages, but only one handler per account
    Oc_EventHandler DeleteGroup GetThreadsReply-$acct
    Oc_EventHandler New _ $acct Readable [list HandleAccountTell $acct] \
            -groups [list $acct GetThreadsReply-$acct]
    set threads [$acct Get]
    Oc_Log Log "Received thread list: $threads" status
    # Create a Net_Thread instance for each element of the returned
    # thread list
    if {![lindex $threads 0]} {
        foreach triple [lrange $threads 1 end] {
            Net_Thread New _ -hostname [$acct Cget -hostname] \
                    -accountname [$acct Cget -accountname] \
                    -pid [lindex $triple 1]
        }
    }
}
# Detect and handle NewThread message from account server
proc HandleAccountTell { acct } {
    set message [$acct Get]
    switch -exact -- [lindex $message 0] {
        newthread {
            NewThread $acct [lindex $message 1]
        }
        default {
          Oc_Log Log "Bad message from account $acct:\n\t$message" status
        }
    }
}
# There's a new thread with id $id, create corresponding local instance
proc NewThread { acct id } {
    Net_Thread New _ -hostname [$acct Cget -hostname] \
            -accountname [$acct Cget -accountname] -pid $id
}
# Delay "nice" of this process until any children are spawned.
Net_Account New a
if {[$a Ready]} {
    AccountReady $a
} else {   
    Oc_EventHandler New _ $a Ready [list AccountReady $a] -oneshot 1
}

vwait forever

