/* FILE: deplock.h                 -*-Mode: c++-*-
 *
 * Dependency lock class.  Protects against deletion of objects
 * while in use by other objects.  Similar to smart pointers,
 * but without the inheritance hierarchy heartaches.  OTOH,
 * this is also not as powerful as smart pointers, either.
 * In particular, objects are not automatically deleted when
 * depcount goes to zero.  There needs to be a distinguished
 * owner responsible for deletion.
 */

#include "oc.h"
#include "oxsexcept.h"
#include "deplock.h"

/* End includes */

Oxs_DepLock::~Oxs_DepLock()
{
  if(depcount>0) 
    OC_THROW(Oxs_BadLock,"Delete with outstanding dependencies");
}

BOOL Oxs_DepLock::ReleaseDependency()
{
  if(depcount<1) return 0;
  --depcount;
  return 1;
}
