/* FILE: deplock.h                 -*-Mode: c++-*-
 *
 * Dependency lock class.  Protects against deletion of objects
 * while in use by other objects.  Similar to smart pointers,
 * but without the inheritance hierarchy heartaches.  OTOH,
 * this is also not as powerful as smart pointers, either.
 * In particular, objects are not automatically deleted when
 * depcount goes to zero.  There needs to be a distinguished
 * owner responsible for deletion.
 */

#ifndef _OXS_DEPLOCK
#define _OXS_DEPLOCK

#include "oc.h"

/* End includes */

// Lock class
class Oxs_DepLock {
private:
  UINT4m depcount;  // Dependency count.  It is okay to delete
  /// this object if depcount==0.

public:
  Oxs_DepLock() : depcount(0) {}
  virtual ~Oxs_DepLock();

  // Copy constructor does not copy depcount
  Oxs_DepLock(const Oxs_DepLock&) : depcount(0) {}

  // Assignment operator does not change depcount
  Oxs_DepLock& operator=(const Oxs_DepLock&) { return *this; }

  UINT4m GetDepCount() const { return depcount; }

  BOOL SetDependency() { depcount++; return 1; }
  BOOL ReleaseDependency();
};

#endif // _OXS_LOCK
