/* FILE: driver.h                 -*-Mode: c++-*-
 *
 * Abstract driver class; fills states and steps evolvers.
 * Also registers the command "Oxs_Run" with the Tcl intepreter.
 *
 */

#ifndef _OXS_DRIVER
#define _OXS_DRIVER

#include "ext.h"
#include "simstate.h"
#include "key.h"

/* End includes */

class Oxs_Evolver; // Forward reference

class Oxs_Driver: public Oxs_Ext {
private:
  Oxs_ConstKey<Oxs_SimState> current_state;
  Oxs_DepKey<Oxs_Evolver> evolver;
  virtual int TclInterface(Tcl_Interp* interp,int argc,char** argv);
  /// This is used to define the Oxs_Run command in the Tcl interpreter.
  /// See also the notes in the Oxs_Ext base class about the use of
  /// this function (and tclcmd_token).
  /// NB: During construction, *EACH* instance of child class of
  /// Oxs_Driver automatically registers an Oxs_Run command in the
  /// Tcl interpreter held by the import Oxs_Director newdtr.  This
  /// command is automatically destroyed during instance deletion.

protected:
  Oxs_Driver(const char* name,        // Child instance id
             Oxs_Director* newdtr,    // App director
             Tcl_Interp* safe_interp, // Safe interpreter
	     const char* argstr);     // Args; should include
                          /// an "evolver <evolver-name>" pair.

  BOOL Init_Driver();  // All children of Oxs_Driver *must* call
  /// this function in their Init() routines.  The main purpose
  /// of this function is to initialize the current state.

public:

  virtual ~Oxs_Driver();

  virtual Oxs_ConstKey<Oxs_SimState> GetInitialState() const =0;

  // Generic interface.
  virtual BOOL Done(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const =0;
  virtual BOOL StageDone(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const =0;
  virtual BOOL Return(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const =0;
  virtual void NewState(const Oxs_SimState& new_state,
    Tcl_Interp*, int, char**) const =0;
  virtual void FillStatePartA(const Oxs_SimState& old_state,
			      Oxs_SimState& new_state) const =0;
  virtual void FillStatePartB(Oxs_SimState& work_state) const =0;
  /// FillStatePartA is called to fill in a new blank state, using
  ///   an old state as a template.
  /// FillStatePartB is called by the evolver to complete a state
  ///   after evolver modifications.
};

#endif // _OXS_DRIVER





