/* FILE: lock.cc                 -*-Mode: c++-*-
 *
 * Data structure locking class
 *
 */

#include "oc.h"
#include "lock.h"
#include "oxsexcept.h"

OC_USING_NAMESPACE_STD;  // Specify std namespace, if supported

/* End includes */

UINT4m Oxs_Lock::id_count=0;

Oxs_Lock::~Oxs_Lock()
{
  if(read_lock>0 || write_lock>0)
    OC_THROW(Oxs_BadLock,"Delete with open locks");
  obj_id=0;
}

// Assignment operator does not copy lock counts, but
// does check access restrictions.  obj_id is copied
// if there are no outstanding locks and obj_id>0.
Oxs_Lock& Oxs_Lock::operator=(const Oxs_Lock& other)
{
  if(read_lock>0)
    OC_THROW(Oxs_BadLock,"Assignment attempt over open read locks");
  if(write_lock==0) {
    // This is the no-lock situation
    if(other.obj_id!=0) {
      obj_id = other.obj_id;
    } else {
      // Take the next valid id
      if((obj_id = ++id_count)==0) {
	// Wrap around.  Might want to issue a warning.
	obj_id = ++id_count;
      }
    }
  }
  // if write_lock>0, then presumably obj_id is already 0.
  return *this;
}

BOOL Oxs_Lock::SetReadLock()
{
  if(write_lock) return 0;
  ++read_lock;
  return 1;
}

BOOL Oxs_Lock::ReleaseReadLock()
{
  if(read_lock<1) return 0;
  --read_lock;
  return 1;
}

BOOL Oxs_Lock::SetWriteLock()
{
  if(read_lock>0 || write_lock>0) return 0;
  write_lock=1;
  obj_id=0;
  return 1;
}

BOOL Oxs_Lock::ReleaseWriteLock()
{
  if(write_lock!=1) return 0;
  if((obj_id = ++id_count)==0) {
    // Wrap around.  Might want to issue a warning.
    obj_id = ++id_count;
  }
  write_lock=0;
  return 1;
}
