/* FILE: lock.h                 -*-Mode: c++-*-
 *
 * Data structure locking class.  For insuring data integrity.
 *
 */

#ifndef _OXS_LOCK
#define _OXS_LOCK

#include "oc.h"

/* End includes */

// Lock class
class Oxs_Lock {
private:
  static UINT4m id_count;  // Top id used.
  UINT4m obj_id; // Object id.  0 is invalid id that is used when a
  /// write lock is outstanding.

  // The following implement normal locking semantics.  You cannot
  // obtain a read_lock if a write_lock is in place.  You cannot
  // obtain a write_lock if a write_lock or one or more read locks
  // are in place.
  UINT4m read_lock;   // Number of read locks
  UINT4m write_lock;  // Number of write locks on this object; 0 or 1
  
public:
  Oxs_Lock() : obj_id(0), read_lock(0), write_lock(0)
    { obj_id = ++id_count; }
  virtual ~Oxs_Lock();
  UINT4m Id() const { return obj_id; }
  BOOL IsLocked() const
  { return (BOOL(read_lock) || BOOL(write_lock)); }

  // Copy constructor does not copy lock counts, but does
  // copy obj_id.
  Oxs_Lock(const Oxs_Lock& other)
    : obj_id(other.obj_id), read_lock(0), write_lock(0)
  {}

  // Assignment operator does not copy lock counts, but
  // does check access restrictions and may copy obj_id.
  Oxs_Lock& operator=(const Oxs_Lock&);

  // The following locking functions return 1 on success, 0 on failure.
  // It is important that any code acquiring a lock release it when
  // done...even if an exception is raised.  Release can be managed
  // automatically by using the Oxs_Key class.  
  BOOL SetReadLock();
  BOOL ReleaseReadLock();
  BOOL SetWriteLock();
  BOOL ReleaseWriteLock();
};

#endif // _OXS_LOCK
