/* FILE: mesh.h                 -*-Mode: c++-*-
 *
 * Abstract Oxs_Mesh class, derives from Oxs_Ext.
 *
 */

#ifndef _OXS_MESH
#define _OXS_MESH

#include "threevector.h"
#include "util.h"
#include "ext.h"

/* End includes */

template<class T> class Oxs_MeshValue;  // Forward declaration

////////////////////////////////////////////////////////////////////////
// Oxs_Mesh (generic mesh) class
class Oxs_Mesh : public Oxs_Ext {
protected:
  // Default file output using generic Oxs_Mesh interface
  int WriteOvfIrregular
  (const char* filename,   // These have same meaning
   const char* title,      // as imports to WriteOvf().
   const char* desc,
   const char* datatype,
   const char* precision,
   const Oxs_MeshValue<ThreeVector>* vec,
   const Oxs_MeshValue<REAL8m>* scale=NULL,
   const ThreeVector* stephints=NULL // If non-null, then the values are
                                   /// written as stephints to the file.
) const;
  Oxs_Mesh(const char* name,Oxs_Director* newdtr);
  Oxs_Mesh(const char* name,Oxs_Director* newdtr,
           Tcl_Interp* safe_interp,const char* argstr);
public:
  virtual ~Oxs_Mesh() {}

  virtual void GetBoundingBox(Oxs_Box& bbox) const =0;
  // Bounding region for mesh

  virtual UINT4m Size() const =0;  // Number of basis elements
  virtual REAL8m TotalVolume() const =0; // Sum of volumes of all basis elts
  virtual REAL8m Volume(UINT4m index) const=0; // Integral of basis elt 'index'
  virtual void Center(UINT4m index,ThreeVector& location) const =0;
  /// Center of mass of basis elt 'index'

  // File output.  Returns 0 on success.  Child classes
  // may use WriteOvfIrregular() above to handle irregular
  // grid output.
  virtual int WriteOvf
  (const char* filename,  // Name of output file
   const char* title,     // Long filename or title
   const char* desc,      // Description to embed in output file
   const char* meshtype,  // Either "rectangular" or "irregular"
   const char* datatype,  // Either "binary" or "text"
   const char* precision, // For binary, "4" or "8";
                          //  for text, a printf-style format
   const Oxs_MeshValue<ThreeVector>* vec,    // Vector array
   const Oxs_MeshValue<REAL8m>* scale=NULL   // Optional scaling for vec
                      /// Set scale to NULL to use vec values directly.
   ) const =0 ;

  // Summing routines.  The terms are weighted by the basis element volumes.
  virtual REAL8m VolumeSum(const Oxs_MeshValue<REAL8m>* scalar) const;
  virtual ThreeVector VolumeSum(const Oxs_MeshValue<ThreeVector>* vec,
				const Oxs_MeshValue<REAL8m>* scale=NULL) const;
  /// "scale" is optional scaling.  Set to NULL to use vec values directly.
};

#endif // _OXS_MESH

