/* FILE: oxsexcept.h                 -*-Mode: c++-*-
 *
 * Exception classes for Oxs.
 *
 */

#ifndef _OXS_EXCEPT
#define _OXS_EXCEPT

#include "oc.h"
#include <iostream>
#include <string>

OC_USING_NAMESPACE_STD;  // Specify std namespace, if supported

/* End includes */

// Dummy declarations that should move over to oommf/ext/oc
#define OC_THROW(except,text) throw except(text)

class Oc_Exception {
private:
  string msg;
public:
  Oc_Exception(string text) : msg(text) {
    cerr << "DEBUG EXCEPTION: " <<  text << '\n';
  }
  virtual ~Oc_Exception() {}
};

// Oxs exceptions
class Oxs_Exception : public Oc_Exception {
public:
  Oxs_Exception(string text) : Oc_Exception(text) {}
};

// Bad pointer
class Oxs_BadPointer : public Oxs_Exception {
public:
  Oxs_BadPointer(string text) : Oxs_Exception(text) {}
};

// Data structure lock action failure
class Oxs_BadLock : public Oxs_Exception {
public:
  Oxs_BadLock(string text) : Oxs_Exception(text) {}
};

// Data structure index invalid
class Oxs_BadIndex : public Oxs_Exception {
public:
  Oxs_BadIndex(string text) : Oxs_Exception(text) {}
};

// Resource allocation failure
class Oxs_BadResourceAlloc : public Oxs_Exception {
public:
  Oxs_BadResourceAlloc(string text) : Oxs_Exception(text) {}
};

// Overflow detected
class Oxs_Overflow : public Oxs_Exception {
public:
  Oxs_Overflow(string text) : Oxs_Exception(text) {}
};

// Bad parameter
class Oxs_BadParameter : public Oxs_Exception {
public:
  Oxs_BadParameter(string text) : Oxs_Exception(text) {}
};

// No memory.  This exists for those broken C++ compilers (you
// know who you are) that don't throw a bad_alloc from new
class Oxs_NoMem : public Oxs_Exception {
public:
  Oxs_NoMem(string text) : Oxs_Exception(text) {}
};


#endif // _OXS_EXCEPT

