/* FILE: threevector.h        -*-Mode: c++-*-
 *
 * Simple 3D vector class.  *All* client code should use the
 * ThreeVector typedef in case this class gets moved into an
 * OOMMF extension library.
 *
 */

#ifndef _OXS_THREEVECTOR
#define _OXS_THREEVECTOR

#include "oc.h"

/* End includes */

class Oxs_ThreeVector;  // Forward declaration for typedef
typedef Oxs_ThreeVector ThreeVector;

class Oxs_ThreeVector {
public:
  REAL8m x,y,z;

  // Constructors
  Oxs_ThreeVector(): x(0.), y(0.), z(0.) {}
  Oxs_ThreeVector(REAL8m xi,REAL8m yi,REAL8m zi)
    : x(xi), y(yi), z(zi) {}
  Oxs_ThreeVector(REAL8m *arr) : x(arr[0]), y(arr[1]), z(arr[2]) {}
  Oxs_ThreeVector(const Oxs_ThreeVector &v) : x(v.x), y(v.y), z(v.z) {}

  // Assignment operators
  Oxs_ThreeVector& Set(REAL8m xi,REAL8m yi,REAL8m zi)
    { x=xi; y=yi; z=zi; return *this; }
  Oxs_ThreeVector& operator=(const Oxs_ThreeVector &v) {
    x=v.x; y=v.y; z=v.z;
    return *this;
  }
  Oxs_ThreeVector& operator+=(const Oxs_ThreeVector& v) { 
    x+=v.x; y+=v.y; z+=v.z;
    return *this;
  }
  Oxs_ThreeVector& operator-=(const Oxs_ThreeVector& v) {
    x-=v.x; y-=v.y; z-=v.z;
    return *this;
  }
  Oxs_ThreeVector& operator^=(const Oxs_ThreeVector& v); // Cross product
  Oxs_ThreeVector& operator*=(REAL8m a) { x*=a; y*=a; z*=a; return *this; }

  // Misc
  REAL8m MagSq() const { return x*x+y*y+z*z; }
  void SetMag(REAL8m mag);  // Adjusts size to "mag"
  void Random(REAL8m mag);  // Makes a random vector of size "mag"

};

// Test operators on Oxs_ThreeVector's
inline BOOL operator==(const Oxs_ThreeVector& lhs,
		const Oxs_ThreeVector& rhs)
{ return ((lhs.x==rhs.x) && (lhs.y==rhs.y) && (lhs.z==rhs.z)); }

inline BOOL operator!=(const Oxs_ThreeVector& lhs,
		const Oxs_ThreeVector& rhs)
{ return ((lhs.x!=rhs.x) || (lhs.y!=rhs.y) || (lhs.z!=rhs.z)); }

// The next two operators are defined so MSVC++ 5.0 will accept
// vector<ThreeVector>, but are left undefined because there
// is no way to define them that makes sense in all contexts.
BOOL operator<(const Oxs_ThreeVector&,const Oxs_ThreeVector&);
BOOL operator>(const Oxs_ThreeVector&,const Oxs_ThreeVector&);


// Binary operators on Oxs_ThreeVector's
inline const Oxs_ThreeVector
operator+(const Oxs_ThreeVector& lhs,
	  const Oxs_ThreeVector& rhs)
{ Oxs_ThreeVector result(lhs); return result+=rhs; }

inline const Oxs_ThreeVector
operator-(const Oxs_ThreeVector& lhs,
	  const Oxs_ThreeVector& rhs)
{ Oxs_ThreeVector result(lhs); return result-=rhs; }

// Cross product
const Oxs_ThreeVector
operator^(const Oxs_ThreeVector& lhs,
	  const Oxs_ThreeVector& rhs);

// Product against scalar
inline const Oxs_ThreeVector
operator*(REAL8m scalar,
	  const Oxs_ThreeVector& vec)
{ Oxs_ThreeVector result(vec); return result*=scalar; }

inline const Oxs_ThreeVector
operator*(const Oxs_ThreeVector& vec,
	  REAL8m scalar)
{ Oxs_ThreeVector result(vec); return result*=scalar; }


// Dot product
inline REAL8m
operator*(const Oxs_ThreeVector& lhs,const Oxs_ThreeVector& rhs)
{ return lhs.x*rhs.x + lhs.y*rhs.y + lhs.z*rhs.z; }

#endif // _OXS_THREEVECTOR
