/* FILE: util.h                 -*-Mode: c++-*-
 *
 * Small utility classes
 *
 */

#ifndef _OXS_UTIL
#define _OXS_UTIL

#include <stdio.h>
#include <string>
#include "oc.h"

OC_USING_NAMESPACE_STD;  // Specify std namespace, if supported

/* End includes */

////////////////////////////////////////////////////////////////////////
// Wrapper for Tcl_SplitList().  This class makes sure memory allocated
// by Tcl_SplitList() is released, even if normal execution path is
// interrupted by an exception.
class Oxs_SplitList
{
private:
  int argc;
  char **argv;
public:
  Oxs_SplitList();
  int Split(const char* argstr);
  int Count() const { return argc; }
  const char* operator[](int n) const;
  ~Oxs_SplitList();
};

////////////////////////////////////////////////////////////////////////
// Wrapper for standard C stream file pointer.  Constructor opens
// file, destructor closes it.
struct Oxs_FilePointer
{
public:
  Oxs_FilePointer(const char* filename,const char* filemode);
  ~Oxs_FilePointer();
  FILE* fptr;
};

////////////////////////////////////////////////////////////////////////
// Simple box class.  Used for defining axes parallel rectangular
// parallelepipeds.
class Oxs_Box {
  REAL8m x0,x1,y0,y1,z0,z1;  // Box corner coordinates.
  /// This Set function insures that x0<=x1, y0<=y1, z0<=z1.
public:
  Oxs_Box() : x0(0.),x1(0.),y0(0.),y1(0.),z0(0.),z1(0.) {}

  void Set(REAL8m x0p,REAL8m x1p,REAL8m y0p,REAL8m y1p,
	   REAL8m z0p,REAL8m z1p);

  void Set(const string& sx0,const string& sx1,
	   const string& sy0,const string& sy1,
	   const string& sz0,const string& sz1);

  REAL8m GetMinX() const { return x0; }
  REAL8m GetMaxX() const { return x1; }
  REAL8m GetMinY() const { return y0; }
  REAL8m GetMaxY() const { return y1; }
  REAL8m GetMinZ() const { return z0; }
  REAL8m GetMaxZ() const { return z1; }
};

#endif // _OXS_UTIL
