/* FILE: bucket.h                    -*-Mode: c++-*-
 *
 * Class for sum accumulation with improved precision
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2000/05/27 06:14:08 $
 * Last modified by: $Author: donahue $
 */

#ifndef _NB_BUCKET
#define _NB_BUCKET

#include "oc.h"
#include "dlist.h"

/* End includes */     /* Optional directive to build.tcl */

//////////////////////////////////////////////////////////////////////////
// Bucket class
//   This is a helper class for BucketAccum.  Should probably be hidden.
class Nb_Bucket
{
private:
  static const ClassDoc class_doc;
  REALWIDE corr;  // Correction term for compensated summation
public:
  int count;
  REALWIDE sum;
  void Reset() { count=0; sum=0.0; corr=0.0; }
  void Spill() { count=0; sum=0.0; } // Empties bucket, but keeps corr.
  Nb_Bucket() { Reset(); }
  void Add(REALWIDE amt) {
    if(amt==0.0) return; // Nothing to do.  Also, we don't want to
    /// increment 'count' needlessly.
    REALWIDE tmp_sum = sum;
    REALWIDE tmp_amt = amt+corr;
    sum    += tmp_amt;
    corr    = (tmp_sum-sum) + tmp_amt;
    count++;
  }
};

//////////////////////////////////////////////////////////////////////////
// BucketAccum (bucket accumulate) class
//   This class provides improved precision for accumulated additions
// by grouping the additions in sets of "accumcount" at a time.  Each
// time "accumcount" items have been added together in one bucket, the
// sum is added to the next bucket up the list.
class Nb_BucketAccum
{
private:
  static const ClassDoc class_doc;
  int accumcount;
  Nb_List<Nb_Bucket> bucket;
public:
  Nb_BucketAccum(int new_accumcount=32):bucket(32)
    { accumcount=new_accumcount; }
  void Clear();
  REALWIDE Accum(REALWIDE newval);
  REALWIDE GetSum();
};

#endif // _NB_BUCKET



