/* FILE: xpfloat.h                    -*-Mode: c++-*-
 *
 * Class for extended floating point precision using
 * compensated summation.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2000/05/05 22:20:59 $
 * Last modified by: $Author: donahue $
 */

#include "oc.h"

#ifndef _NB_XPFLOAT_H
#define _NB_XPFLOAT_H

class Nb_Xpfloat
{
private:
  REALWIDE x,corr;
public:
  Nb_Xpfloat(REALWIDE newx=0.0) { x=newx; corr=0.; }
  Nb_Xpfloat(const Nb_Xpfloat& y) { x=y.x; corr=y.corr; }
  void Set(REALWIDE newx) { x=newx; corr=0.; }
  REALWIDE GetValue() const { return x; }
  void Accum(REALWIDE y);
  void Accum(const Nb_Xpfloat& y);
  Nb_Xpfloat& operator+=(const Nb_Xpfloat& y) { Accum(y); return *this; }
  Nb_Xpfloat& operator-=(const Nb_Xpfloat& y);
  Nb_Xpfloat& operator*=(REALWIDE y);
  operator REALWIDE() { return GetValue(); }
  friend Nb_Xpfloat operator*(const Nb_Xpfloat& x,const Nb_Xpfloat& y);
};

Nb_Xpfloat operator+(const Nb_Xpfloat& x,const Nb_Xpfloat& y);
Nb_Xpfloat operator-(const Nb_Xpfloat& x,const Nb_Xpfloat& y);
Nb_Xpfloat operator*(const Nb_Xpfloat& x,REALWIDE y);
inline Nb_Xpfloat operator*(REALWIDE y,const Nb_Xpfloat& x) { return x*y; }
Nb_Xpfloat operator*(const Nb_Xpfloat& x,const Nb_Xpfloat& y);

#endif // _NB_XPFLOAT_H
